#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>

#include "protocollo.h"

// Stampa a vide un menu con le operazioni possibili
// Per semplicita' solo la prima e l'ultima sono implementate
void printMenu() {
  const char* menu = "\n-------------------\n"
                     "1) scatta una foto\n"
                     "2) ...\n"
                     "0) esci\n"
                     "? ";
  printf("%s",menu);
}

/* Invia un comando come uint16
 * Restituisce un valore negativo in caso di fallimento in trasmissione.
 */
int inviaComando(int fd, uint16_t v) {
  uint16_t comando;
  int ret;

  // Converte in formato rete
  comando = htons(v);
  // Invia tramite il socket fd
  ret = send(fd, (void *) &comando, sizeof(comando), 0);
  if (ret < 0 || ret < sizeof(comando)){  /* error */
    if (ret < 0) {
        perror("send");
    }
    return -1;
  }
  return ret;
}

/* Restituisce l'esito.
 * Se negativo c'è stato un fallimento in ricezione.
 */
int leggiEsito(int cn_fd) {
  uint16_t esito;
  int ret;

  // Riceve un uint16 dal socket cn_fd
  ret = recv(cn_fd, (void *) &esito, sizeof(esito), MSG_WAITALL);
  if (ret < 0 || ret < sizeof(esito)) { /* error */
      if (ret < 0) {
          perror("recv(len)");
      }
      return -1;
  }
  // Converte in formato locale
  esito = ntohs(esito);
  return esito;
}

int main (int argc, char *argv[])
{
    struct sockaddr_in srv_addr;
    int ret, fd;
    int c;
    int ancora = 1;
    uint16_t e;

    if (argc < 3) {
        printf("Wrong number of parameters!\n");
        printf("USAGE: %s [SERVER_IP] [PORT]\n", argv[0]);
        exit(1);
    }

    // Crea il socket
    fd = socket(AF_INET, SOCK_STREAM, 0);
    // Azzera la struttura
    memset(&srv_addr, 0, sizeof(srv_addr));
    // Protocollo IP
    srv_addr.sin_family = AF_INET;
    // Converte la porta da stringa
    srv_addr.sin_port = htons(atoi(argv[2]));
    // Converte l'indirizzo da stringa
    ret = inet_pton(AF_INET, argv[1], &srv_addr.sin_addr);

    // Si connette al server
    ret = connect(fd, (struct sockaddr *) &srv_addr, sizeof(srv_addr));
    if (ret < 0){  /* error */
        perror("connect()");
        return ret;
    }

    do {
        // Stampa il menu
        printMenu();
        // Legge la scelta dell'utente
        if(!scanf("%d", &c)) {
          printf("Scelta non valida\n");
          getchar();
          continue;
        }


        switch(c) {
          case 1:
            // Invia il comando al server
            inviaComando(fd, FOTO_COLORI);
            e = leggiEsito(fd);
            if(e == OK) printf("Foto acquisita\n");
            else printf("Problema...\n");
            break;
          case 2:
            // Non implementato...
            break;
          case 0:
            // Manda messaggio di chiusura
            inviaComando(fd, FINE);
            e = leggiEsito(fd);
            if(e == OK) printf("Ho chiuso.\n");
            else printf("Problema...\n");
            ancora = 0;
            break;
        }
    } while (ancora);

    close(fd);
    return 0;
}
