#include <dirent.h>
#include <stdio.h>
#include <string.h>

/*
 * Header di una semplice pagina HTML
 */
static const char * page_header = "<html>"
                           " <head>"
                           "  <title> Esempio Raspberry </title>"
                           " </head>"
                           " <body>";

/*
 * Footer di una semplice pagina HTML
 */
static const char * page_footer = " </body>"
                           "</html>";

/*
 * Per ognuna delle immagini jpg prodotte dalla fotocamera viene prodotto un
 * paragrafo contenente un link all'immagine stessa.
 * Il numero è limitato a 10.
 * pf e' il file index.html di cui bisogna produrre il corpo
 * x e' il path della cartella
 */
static int produciCorpo(FILE* pf, const char* x) {
  // Cartella che contiene le immagini jpg
  DIR *dd;
  // Struttura usata per ottenere i nomi dei file contenuti nella cartella
  struct dirent * elem;
  // Contatore delle immagini jpg
  int conta = 0;

  // Apre la cartella
  dd = opendir(x);
  if (dd == NULL)
    return -1;

  // Scorre gli elementi contenuti nella cartella
  while ((elem = readdir(dd)) != NULL && conta < 10) {
    // Se il file termina per jpg lo dobbiamo includere
    if(strstr(elem->d_name, ".jpg")) {
      // Crea un paragrafo contenente il link all'immagine e lo scrive nel file
      fprintf(pf, "<p><img src=\"%s\" style=\"width:500px\" /></p>\n", elem->d_name);
      conta++;
    }
  }
  // Chiude la cartella
  closedir(dd);
  // Restituisce il numero di immagini incluse nel file
  return conta;
}

/*
 * Crea una pagina index.html nella cartella indicata dal path p.
 * La pagina contiene al piu' 10 immagini jpg prodotte dalla
 * fotocamera e salvate sempre nella cartella p.
 */
int produciPagina(const char* p) {
  // File index.html
  FILE * pFile;
  // Path del file index.html
  char name [1024];
  // Valore di ritorno
  int ret;

  // Copia il path della cartella in name
  snprintf(name, sizeof(name), "%s", p);
  // Appende in fondo index.html
  strncat(name, "index.html", sizeof(name)-strlen(p));
  // Apre il file in scrittura
  pFile = fopen (name,"w");
  if(!pFile) {
    perror("fopen");
    return -1;
  }
  // Scrive nel file l'header della pagine HTML
  fprintf(pFile, "%s", page_header);
  // Scrive nel file il corpo della pagina, che contiene i link alle immagini
  ret = produciCorpo(pFile, p);
  if(ret<0) {
    perror("produciCorpo");
    return -1;
  }
  // Scrive nel file il footer della pagina HTML
  fprintf(pFile, "%s", page_footer);
  // Chiude il file
  fclose(pFile);
  return 0;
}
